from django.urls import path

from .views import (
    PaymentInitFormView,
    SuccessView,
    FailView,
    PaymentInitApiView,
    PaidNotificationView,
    CheckNotificationView,
)
from .forms import PaymentForm

app_name = "moneta"

urlpatterns = [
    path(
        "check_notification",
        CheckNotificationView.as_view(),
        name="check-notification",
    ),
    path("paid_notification", PaidNotificationView.as_view(), name="paid-notification"),
    path(
        "init_pay",
        PaymentInitFormView.as_view(
            form_class=PaymentForm,
            template_name="moneta/init.html",
        ),
        name="init",
    ),
    path(
        "success",
        SuccessView.as_view(template_name='moneta/success.html'),
        name="payment-success",
    ),
    path(
        "fail",
        FailView.as_view(template_name='moneta/fail.html'),
        name="payment-fail",
    ),
    path(
        "api_init_pay/<int:order_id>/<int:amount>",
        PaymentInitApiView.as_view(),
        name="api-init",
    ),
]



# Generated by Django 3.1.7 on 2021-04-10 20:14
import os
from sys import path

from django.db import migrations
from django.core import serializers


fixture_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), "../fixtures"))
fixture_filename = "payment_systems.json"


def load_fixtures(apps, schema_editor):
    fixture_file = os.path.join(fixture_dir, fixture_filename)
    with open(fixture_file, "rb") as fib:
        objects = serializers.deserialize("json", fib, ignorenonexistent=True)
        for obj in objects:
            obj.save()


def unload_fixtures(apps, schema_editor):
    PaymentSystem = apps.get_model("moneta", "PaymentSystem")
    PaymentSystem.objects.all().delete()



class Migration(migrations.Migration):

    dependencies = [
        ('moneta', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(load_fixtures, reverse_code=unload_fixtures)
    ]



from typing import Optional
from django.db import models


class PaymentSystem(models.Model):
    ELECTRONIC = "electronic"
    BANK = "bank"
    GROUP_CHOICES = ((ELECTRONIC, ELECTRONIC), (BANK, BANK))

    group = models.CharField(max_length=10, choices=GROUP_CHOICES)
    title = models.CharField(max_length=50)
    name = models.CharField(max_length=30)
    account_id = models.PositiveIntegerField()
    unit_id = models.CharField(max_length=100)
    create_invoice = models.BooleanField()

    def __str__(self) -> str:
        return f"PaymentSystem ({self.name})"

    class Meta:
        db_table = "mnt_payment_systems"


class InvoiceManager(models.Manager):
    def create_check_invoice(self, transaction_id: str, values: dict) -> "Invoice":
        return self.get_or_create(transaction_id=transaction_id, defaults=values)

    def finalize(self, transaction_id: str) -> Optional["Invoice"]:
        invoice = self.filter(transaction_id=transaction_id).last()
        if invoice is None:
            return None
        invoice.status = Invoice.PAID
        invoice.save(update_fields=["status"])
        return invoice


class Invoice(models.Model):
    CHECK = "CHECK"
    PAID = "PAID"
    status_choices = ((CHECK, CHECK), (PAID, PAID))
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    merchant_id = models.CharField(help_text="MNT_ID", max_length=30)
    transaction_id = models.CharField(max_length=255, unique=True)
    operation_id = models.CharField(
        max_length=30, help_text="MNT_OPERATION_ID", blank=True, null=True
    )
    amount = models.DecimalField(
        max_digits=11, decimal_places=2, help_text="MNT_AMOUNT"
    )
    description = models.CharField(max_length=200, blank=True)
    currency = models.CharField(max_length=3, help_text="MNT_CURRENCY_CODE")
    payment_system = models.ForeignKey(
        "PaymentSystem", on_delete=models.SET_NULL, blank=True, null=True
    )
    test_mode = models.BooleanField(help_text="MNT_TEST_MODE")
    signature = models.CharField(max_length=250, help_text="MNT_SIGNATURE", blank=True)
    payment_system_unit_id = models.CharField(
        max_length=10, help_text="paymentSystem.unitId", blank=True
    )
    corraccount = models.CharField(
        max_length=20, help_text="MNT_CORRACCOUNT", blank=True, null=True
    )
    subscriber_id = models.CharField(
        help_text="MNT_SUBSCRIBER_ID", blank=True, max_length=200
    )
    status = models.CharField(max_length=6, choices=status_choices, default=CHECK)

    objects = InvoiceManager()

    def __str__(self) -> str:
        return f"Invoice ({self.id})"

    class Meta:
        db_table = "mnt_invoices"



import urllib.parse
from django.conf import settings as django_settings
from django.utils.functional import SimpleLazyObject
from pydantic import BaseModel, HttpUrl, validator
from typing import Optional, Union

from moneta.models import PaymentSystem


class BasicConfig(BaseModel):
    demo_mode: Optional[bool] = True
    test_mode: Optional[bool] = True
    debug_mode: Optional[bool] = True
    account_id: int
    account_code: Optional[str] = None
    account_username: str
    account_password: str
    prototype_user_unit_id: Optional[str] = None
    history_items_per_page: Optional[int] = 20
    currency: Optional[str] = "RUB"
    payment_description: Optional[str] = None
    payment_system_name: Optional[str] = "plastic"
    signature_required: Optional[bool] = False

    def get_account_code(self) -> str:
        if self.account_code is None:
            return ""
        return str(self.account_code)

    def get_test_mode(self) -> str:
        return str(int(self.test_mode))

    @validator("payment_system_name")
    def checkin_payment_system_id(cls, v):
        if not PaymentSystem.objects.filter(name=v).exists():
            raise ValueError("payment system not found")
        return v


class UrlsConfig(BaseModel):
    demo_url: Optional[HttpUrl] = "https://demo.moneta.ru"
    production_url: Optional[HttpUrl] = "https://service.moneta.ru"
    success_url: Optional[str] = None
    fail_url: Optional[str] = None
    inprogress_url: Optional[str] = None
    return_url: Optional[str] = None
    iframe_target: Optional[str] = "_parent"
    assistant_link: Optional[str] = "/assistant.htm"
    soap_link: Optional[str] = "/services.wsdl"
    json_link: Optional[str] = "/services"
    assistant_widget_link: Optional[str] = "assistant.widget"
    x509_port: Optional[int] = 8443
    x509_soap_link: Optional[str] = "/services/x509.wsdl"
    x509_json_link: Optional[str] = "/services/x509"


class MainConfig(BaseModel):
    basic_config: BasicConfig
    urls_config: UrlsConfig

    @property
    def payment_url(self) -> str:
        if self.basic_config.demo_mode:
            return self.urls_config.demo_url
        return self.urls_config.production_url

    @property
    def soap_url(self) -> str:
        return urllib.parse.urljoin(self.payment_url, self.urls_config.soap_link)


    def get_payment_description(self) -> Optional[str]:
        return self.basic_config.payment_description

    @property
    def payment_system(self) -> PaymentSystem:
        pay_system = PaymentSystem.objects.filter(
            name=self.basic_config.payment_system_name
        ).last()
        return pay_system

    def __getattr__(self, item):
        return getattr(self.basic_config, item)


DEFAULTS = {
    "basic_config": {},
    "urls_config": {},
}


class MainSettings:
    def __init__(self, user_settings: dict, defaults=None):
        if defaults is None:
            defaults = DEFAULTS
        config = {**defaults, **user_settings}
        self._config = MainConfig(**config)

    def __getattr__(self, attr: str) -> Union[str, int]:
        return getattr(self._config, attr)

    def config(self) -> MainConfig:
        return self._config

    def url_generator(self) -> "UrlGenerator":
        from .url_generator import UrlGenerator

        return UrlGenerator(config=self.config())


class LazyWrapperMonetaConfig:
    def __init__(self):
        self.settings = None

    def __call__(self):
        if self.settings is None:
            self.settings = MainSettings(
                user_settings=getattr(django_settings, "MONETA", {})
            )

    def __getattr__(self, attr: str):
        if self.settings is None:
            self.settings = MainSettings(
                user_settings=getattr(django_settings, "MONETA", {})
            )
            return getattr(self.settings, attr)


def _init_config():
    return MainSettings(user_settings=getattr(django_settings, "MONETA", {}))


moneta_config = SimpleLazyObject(_init_config)



from hashlib import md5
from pydantic import BaseModel
from typing import Optional, Union

from moneta.models import PaymentSystem
from .config import MainConfig
from moneta.utils import add_url_query_params


class QueryPayload(BaseModel):
    account_id: str  # MNT_ID
    transaction_id: str  # MNT_TRANSACTION_ID
    amount: str  # MNT_AMOUNT
    currency: str  # MNT_CURRENCY_CODE
    test_mode: str  # MNT_TEST_MODE
    unit_id: str  # paymentSystem.unitId
    limit_ids: str  # paymentSystem.limitIds
    subscriber_id: Optional[str] = None  # MNT_SUBSCRIBER_ID
    success_url: Optional[str] = None
    description: Optional[str] = None
    signature: Optional[str] = None

    class Config:
        allow_population_by_field_name = True
        fields = {
            "account_id": "MNT_ID",
            "transaction_id": "MNT_TRANSACTION_ID",
            "amount": "MNT_AMOUNT",
            "currency": "MNT_CURRENCY_CODE",
            "test_mode": "MNT_TEST_MODE",
            "unit_id": "paymentSystem.unitId",
            "limit_ids": "paymentSystem.limitIds",
            "success_url": "MNT_SUCCESS_URL",
            "description": "MNT_DESCRIPTION",
            "signature": "MNT_SIGNATURE",
            "subscriber_id": "MNT_SUBSCRIBER_ID",
        }


class UrlGenerator(BaseModel):
    config: MainConfig

    @property
    def root_url(self) -> str:
        return f"{self.config.payment_url}{self.config.urls_config.assistant_link}"

    def _signature_chunks(
        self, order_id: str, amount: float, subscriber_id: str
    ) -> str:
        signature_chunks = [
            str(self.config.basic_config.account_id),
            order_id,
            self.format_amount(amount),
            self.config.basic_config.currency,
            subscriber_id,
            self.config.basic_config.get_test_mode(),
            self.config.basic_config.get_account_code(),
        ]
        return "".join(signature_chunks)

    def signature(
        self, order_id: Union[str, int], amount: float, subscriber_id: str
    ) -> Optional[str]:
        if self.config.basic_config.account_code is None:
            return None
        sign = self._signature_chunks(str(order_id), amount, subscriber_id)
        return md5(sign.encode("utf-8")).hexdigest()

    @staticmethod
    def format_amount(amount: float) -> str:
        return f"{round(amount, 2):.2f}"

    def payment_url(
        self,
        order_id: str,
        amount: float,
        subscriber_id: Optional[str] = "",
        payment_system: Optional[PaymentSystem] = None,
        description: Optional[str] = None,
    ) -> str:
        pay_sys = payment_system if payment_system else self.config.payment_system
        order_id = str(order_id)
        query = QueryPayload(
            account_id=str(self.config.basic_config.account_id),
            transaction_id=order_id,
            amount=self.format_amount(amount),
            currency=self.config.basic_config.currency,
            test_mode=self.config.basic_config.get_test_mode(),
            unit_id=pay_sys.unit_id,
            limit_ids=pay_sys.unit_id,
            success_url=self.config.urls_config.success_url,
            description=description
            if description
            else self.config.get_payment_description(),
            signature=self.signature(order_id, amount, subscriber_id),
            subscriber_id=subscriber_id,
        )
        payload = query.dict(by_alias=True, exclude_none=True)
        return add_url_query_params(url=self.root_url, additional_params=payload)

    class Config:
        arbitrary_types_allowed = True



from decimal import Decimal
from hashlib import md5

from pydantic import BaseModel, Field
from typing import Optional

from moneta.models import PaymentSystem
from moneta.configuration import moneta_config


class MonetaQueryParameters(BaseModel):
    """
    check parameters
     https://demo.moneta.ru/doc/MONETA.Assistant.ru.pdf
    """

    # command: str = Field(alias="MNT_COMMAND")
    merchant_id: int = Field(alias="MNT_ID")
    transaction_id: str = Field(alias="MNT_TRANSACTION_ID")
    currency: str = Field(alias="MNT_CURRENCY_CODE")
    test_mode: str = Field(alias="MNT_TEST_MODE")
    signature: str = Field(alias="MNT_SIGNATURE")
    subscriber_id: Optional[str] = Field(alias="MNT_SUBSCRIBER_ID", default=None)
    amount: Optional[Decimal] = Field(alias="MNT_AMOUNT")
    operation_id: Optional[str] = Field(alais="MNT_OPERATION_ID")
    moneta_user: Optional[str] = Field(alias="MNT_USER")
    unit_id: Optional[str] = Field(alias="paymentSystem.unitId", default=None)
    corraccount: Optional[str] = Field(alias="MNT_CORRACCOUNT")

    def get_payment_system(self) -> Optional[PaymentSystem]:
        return PaymentSystem.objects.filter(unit_id=self.unit_id).last()

    def get_test_mode(self) -> bool:
        return bool(int(self.test_mode))

    @staticmethod
    def result_code() -> int:
        """
        MONETA RESULT CODES:
            - 100: Answer contains amount field. This code should use when check query doen not contain MNT_AMONT
            - 200: Order paid. Notification about paid delivered.
            - 302: Order processing.
            - 402: Order created and ready to pay. Pay notification did not deliver
            - 500: Order outdated. Pay process will be ended.

            In normal cases merchant should return 402 or 100 codes
        """
        return 402

    def get_signature(self, account_code: Optional[str] = None) -> str:
        if account_code is None:
            account_code = moneta_config.get_account_code()
        chunks = [
            str(self.result_code()),
            str(self.merchant_id),
            str(self.transaction_id),
            account_code,
        ]
        sign = "".join(chunks)
        return md5(sign.encode("utf-8")).hexdigest()

    def xml_body(self, account_code: Optional[str] = None) -> str:
        return f"""<?xml version="1.0" encoding="UTF-8"?>
        <MNT_RESPONSE>
            <MNT_ID>{self.merchant_id}</MNT_ID>
            <MNT_TRANSACTION_ID>{self.transaction_id}</MNT_TRANSACTION_ID>
            <MNT_RESULT_CODE>{self.result_code()}</MNT_RESULT_CODE>
            <MNT_DESCRIPTION></MNT_DESCRIPTION>
            <MNT_AMOUNT>{self.amount}</MNT_AMOUNT>
            <MNT_SIGNATURE>{self.get_signature(account_code)}</MNT_SIGNATURE>
        </MNT_RESPONSE>
                """
